#ifndef SKILLEXECUTIONTREEWIDGET_H
#define SKILLEXECUTIONTREEWIDGET_H

#include <QTreeWidget>

#include <ArmarXCore/core/time.h>

#include "../memory/MemoryCommunicatorBase.h"

namespace armarx::skills::gui
{

    // cannot be declared in class -> in namespace
    static const std::map<skills::SkillStatus, std::string> EXECUTION_STATUS_TO_STRING = {
        // Main states
        {skills::SkillStatus::Constructing, "Constructing"},
        {skills::SkillStatus::Initializing, "Initializing"},
        {skills::SkillStatus::Preparing, "Preparing"},
        {skills::SkillStatus::Running, "Running"},

        // Terminating
        {skills::SkillStatus::Aborted, "Aborted"},
        {skills::SkillStatus::Failed, "Failed"},
        {skills::SkillStatus::Succeeded, "Succeeded"}};

    class SkillExecutionTreeWidget : public QTreeWidget, public MemoryCommunicatorBase
    {
    public:
        SkillExecutionTreeWidget(std::shared_ptr<SkillManagerWrapper> _memory,
                                 QWidget* parent = nullptr) :
            QTreeWidget(parent), MemoryCommunicatorBase(_memory)
        {
            setupUi();
        }

        /**
         * @brief Describes the currently selected execution in the tree.
         */
        struct SelectedExecution
        {
            skills::SkillExecutionID skillExecutionId;

            // make default constructable
            SelectedExecution() :
                skillExecutionId{
                    .skillId = {.providerId = std::nullopt, .skillName = skills::SkillID::UNKNOWN},
                    .executorName = skills::SkillExecutionID::UNKNOWN,
                    .executionStartedTime = armarx::core::time::DateTime::Invalid()}
            {
            }

            /**
             * @brief Checks the validity of the selected item at run time.
             * @return the validity.
             */
            bool valid();
        };

        SelectedExecution& getSelectedExecution();

    public slots:

        /**
         * @brief Clears all state of this widget.
         */
        void disconnectGui();
        /**
         * @brief Fetches an update from memory and applies it to this widget.
         */
        void updateExecutions();

    private slots:
        void executionSelectionChanged(QTreeWidgetItem* current, QTreeWidgetItem* previous);
        void runContextMenu(const QPoint& pos);
        void stopSelectedExecution();
        void rerunSkillWithSimilarParams();

    private:
        void setupUi();
        void connectSignals();

        SelectedExecution selectedExecution;
    };
} // namespace armarx::skills::gui

#endif // SKILLEXECUTIONTREEWIDGET_H
